#include <windows.h>
#include <stdio.h>
#include "FTD2XX.H"

//////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************/
/* Structures                                                           */
/************************************************************************/

typedef struct _RF620_HELLO_ANSWER_{
	
	WORD	wDeviceType;
	BYTE	bDeviceModificaton;
	DWORD	dwDeviceSerial;
	WORD	wBeginZRange;
	WORD	wZRange;
	WORD	wX_SMR;
	WORD	wX_EMR;

} RF620HELLOANSWER, *LPRF620HELLOANSWER;


typedef struct _RF620_IMAGE_INFO {

	WORD	wLeft;
	WORD	wTop;
	WORD	wWidth;
	WORD	wHeight;

} RF620_IMAGE_INFO, *LPRF620_IMAGE_INFO;


typedef struct _RF620_RESULT_INFO {

	BYTE	bNumPack;
	WORD	wNumPoint;

} RF620_RESULT_INFO, *LPRF620_RESULT_INFO;

//////////////////////////////////////////////////////////////////////////

/************************************************************************/
/* Definitions                                                          */
/************************************************************************/

///////////////////////////////////////////////////////// Address / Length ////////////////
#define	RF620_PARAMETER_LASER_POWER_STATE				((0x01) | ((0x01) << 8))
#define	RF620_PARAMETER_NETWORK_ADDRESS					((0x02) | ((0x01) << 8))
#define	RF620_PARAMETER_BAUDRATE						((0x03) | ((0x01) << 8))
#define	RF620_PARAMETER_X_POINTS_COUNT					((0x04) | ((0x01) << 8))
#define	RF620_PARAMETER_LASER_INTENSITY					((0x05) | ((0x01) << 8))
#define	RF620_PARAMETER_EXT_SYNC						((0x06) | ((0x01) << 8))
#define	RF620_PARAMETER_SYNC_DEVIDER					((0x07) | ((0x02) << 8))
#define	RF620_PARAMETER_RESULT_HOLD_TIME				((0x08) | ((0x02) << 8))
#define	RF620_PARAMETER_WINDOW_BORDER_H					((0x09) | ((0x02) << 8))
#define	RF620_PARAMETER_WINDOW_BORDER_V					((0x0A) | ((0x02) << 8))
#define	RF620_PARAMETER_POINT_HOLD_TIME					((0x0B) | ((0x02) << 8))
#define	RF620_PARAMETER_EXPOSURE_TIME					((0x0C) | ((0x02) << 8))
#define	RF620_PARAMETER_LASER_DELAY						((0x0D) | ((0x02) << 8))

//////////////////////////////////////////////////////////////////////////

/************************************************************************/
/* Functions                                                            */
/************************************************************************/

/************************************************************************/
/* RS-485 functions                                                     */
/************************************************************************/

BOOL	RF620_OpenPort(			
			LPCSTR					lpPort_Name, 
			DWORD					dwSpeed, 
			FT_HANDLE *				lpftHandle 
			);

BOOL	RF620_ClosePort( 
			FT_HANDLE				hftHandle 
			);

BOOL	RF620_HelloCmd(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress, 
			LPRF620HELLOANSWER		lprfHelloAnswer 
			);

BOOL	RF620_ReadParameter(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress, 
			WORD					wParameter,
			USHORT *				lpusValue 
			);

BOOL	RF620_WriteParameter(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress, 
			WORD					wParameter,
			USHORT					usValue 
			);

BOOL	RF620_FlushParamsToFlash(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress
			);

BOOL	RF620_RestoreParamsFromFlash(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress
			);

BOOL	RF620_GetMeasure(	
			FT_HANDLE				hCOM, 
			BYTE					bAddress, 
			LPRF620_RESULT_INFO		lpResInfo,
			USHORT *				lpusValue 
			);

BOOL	RF620_GetImage(	
			FT_HANDLE				hCOM, 
			BYTE					bAddress, 
			LPRF620_IMAGE_INFO		lprfImageInfo,
			BYTE *					lpbImage 
			);

BOOL	RF620_LockResult(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress
			);

BOOL	RF620_RotateAxisXY(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress,
			FLOAT *					lpfAngle
			);

BOOL	RF620_GetRotateAngle(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress,
			FLOAT *					lpfAngle
			);

BOOL	RF620_FlushAngleToFlash(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress
			);

BOOL	RF620_RestoreAngleFromFlash(		
			FT_HANDLE				hCOM, 
			BYTE					bAddress
			);

#ifdef __cplusplus
}
#endif      

//////////////////////////////////////////////////////////////////////////