//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Main.h"
#include "Math.h"
#include "RF620Interface.h"
#include "MotorInterface.h"
#include "ExtSig.h"
#include "Exposure.h"
#include "Exp_delay.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "CSPIN"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TForm_Main *Form_Main;
//---------------------------------------------------------------------------
TRF620Interface *RF620Interface;
RF620INT_STATUS RF620_status;
RF620INTPARAMETERS RF620INT_Parameters;
tHELLO_REGs_MAP HELLO_PARAMETERS;
tFACT_REGs_MAP FACTORY_PARAMETERS, FACTORY_PARAMETERS_RD;
tUSER_REGs_MAP USER_PARAMETERS, USER_PARAMETERS_RD;
BYTE ind_User_Param, ind_Factory_Param;
WORD RESULT_BUFFER[4096];
BYTE BITMAP_BUFFER[con_BMP_Width*con_BMP_Height];
Graphics::TBitmap *pBitMap;
BYTE Bmp_Loading_Progress;
TRect ROI_Window, ROI_Window_Old;
bool Set_ROI;
float fAngle;
WORD FrameFreq;
//---------------------------------------------------------------------------
TMotorInterface *MOTORInterface;
MOTORINT_STATUS MOTOR_status;
MOTORINTPARAMETERS MOTORINT_Parameters;
WORD CALIBR_BUFFER[con_TBL_Width][con_DIAP_Z_Steps];
WORD CALIBR_BUFFER_PXL[con_TBL_Width][con_DIAP_Z_Steps];
short int CALIBR_BUFFER_Z[con_TBL_Width][10 * con_TBL_Height];
short int CALIBR_BUFFER_X[con_TBL_Width][10 * con_TBL_Height];
short int CALIBR_BUFFER_TABLES[2 * con_TBL_Width * con_TBL_Height];
BYTE CALIBR_BUFFER_NMEAS[con_TBL_Width];
AnsiString CurrDirectory;
float delta_Um;
WORD nSteps, ind_Step;
float DATA_TO_FILE[1024][2];
WORD DATA_TO_FILE_Pnt = 1024;
//---------------------------------------------------------------------------
__fastcall TForm_Main::TForm_Main(TComponent* Owner)
        : TForm(Owner)
{
        RF620Interface = new TRF620Interface;
        MOTORInterface = new TMotorInterface;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::FormCreate(TObject *Sender)
{
        ComboBox_Connect_NetAddress->Clear();
        for (BYTE i = 0; i <= con_cNET_ADDR; i++)
                ComboBox_Connect_NetAddress->AddItem(IntToStr(i), Sender);
        ComboBox_Connect_NetAddress->ItemIndex = 0;
        ComboBox_Connect_Baud->ItemIndex = 7;
        Form_Main->Constraints->MinHeight = Min_Height = (Form_Main->Height - (Panel_Parameters->Height + Panel_Connect->Height + Panel_Ranges->Height)) + (Panel_Connect->Height + Panel_Ranges->Height);
        Form_Main->Height = Form_Main->Constraints->MinHeight;
        PaintBox_BitMap->Left = 0;
        PaintBox_BitMap->Top = 0;
        PaintBox_BitMap->Width = con_BMP_Width;
        PaintBox_BitMap->Height = con_BMP_Height;
        Timer_Resize->Enabled = false;
        Timer_Result->Enabled = false;
        Timer_Motor->Enabled = false;
        ind_User_Param = ind_Factory_Param = 0;
        pBitMap = new Graphics::TBitmap();
        pBitMap->Width = con_BMP_Width;
        pBitMap->Height = con_BMP_Height;
        pBitMap->PixelFormat = pf8bit;
        LOGPALETTE *pal = (LOGPALETTE*) new char[sizeof(LOGPALETTE) + 256*4];
        pal->palVersion = 0x300;
        pal->palNumEntries = 256;
        for (int i = 0; i < 256; i++) {
                pal->palPalEntry[i].peFlags = 0;
                pal->palPalEntry[i].peRed = i;
                pal->palPalEntry[i].peGreen = pal->palPalEntry[i].peBlue = 0;
        }
        pBitMap->Palette = CreatePalette(pal);
        for (WORD y = 0; y < pBitMap->Height; y++) {
                BYTE *ptr_In = (BYTE*) pBitMap->ScanLine[y];
                for (WORD x = 0; x < pBitMap->Width; x++)
                        ptr_In[x] = 0;
        }
        delete [] pal;
        CurrDirectory = GetCurrentDir();
        fAngle = 0.0;
        Panel_Angle->Caption = "Angle = "+ FloatToStrF(fAngle * 180 / M_PI, ffFixed, 3, 3) + "";
        RF620_status = RF620INT_OK;
        Button_Get_Result->Enabled = false;
        Button_Get_Video->Enabled = false;
        Timer_Repaint->Enabled = false;
        Timer_Errors->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::FormResize(TObject *Sender)
{
        Form_Main->Height = Form_Main->Constraints->MinHeight;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::FormClose(TObject *Sender,
      TCloseAction &Action)
{
        RF620Interface->Disonnect();
        MOTORInterface->Disonnect();
        delete pBitMap;
        delete MOTORInterface;
        delete RF620Interface;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::View_User_Parameters()
{
        BYTE ind_Par;
        WORD Value;
        if (ValueListEditor_User_Parameters->RowCount == con_USER_REGISTERS + 1) {
                for (ind_Par = 0; ind_Par < con_USER_REGISTERS; ind_Par++) {
                        Value = USER_PARAMETERS.REG_ADDR[ind_Par];
                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                        switch (ind_Par){
                                case ra_USER_POWER:
                                        if (Value) ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = "on";
                                        else ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = "off";
                                break;
                                case ra_USER_ADDR:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_RSBR:
                                        if ((Value & 0x40) == 0x00) ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(115200 << Value);
                                        else ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(4800 << Value);
                                break;
                                case ra_USER_RES_LEN:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(128 << Value);
                                break;
                                case ra_USER_LAS_INT:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_EXT_PORT:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_EXT_DIV:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value + 1);
                                break;
                                case ra_USER_RES_WAIT:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(5 * Value);
                                break;
                                case ra_USER_WND_LW:
                                      ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_WND_TH:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_PNT_TIME:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(2 * Value + 10);
                                break;
                                case ra_USER_EXPOSURE:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_EXPOS_DELAY:
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;

                        }
                }
        } else {
                for (ind_Par = 0; ind_Par < con_USER_REGISTERS; ind_Par++) {
                        Value = USER_PARAMETERS.REG_ADDR[ind_Par];
                        ValueListEditor_User_Parameters->InsertRow(str_USER_PARAMETERS_NAME[ind_Par], IntToStr(Value), true);
                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->EditStyle = es_USER_PARAMETERS[ind_Par];
                        switch (ind_Par){
                                case ra_USER_POWER:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 3;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->CommaText = "off,on";
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        if (Value) ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = "on";
                                        else ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = "off";
                                break;
                                case ra_USER_ADDR:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 2;
                                        for (BYTE j = 0; j <= con_cNET_ADDR; j++)
                                                ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->Add(IntToStr(j));
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                break;
                                case ra_USER_RSBR:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 6;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->CommaText = "4800,9600,19200,38400,115200,230400,460800,921600";
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        if ((Value & 0x40) == 0x00) ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(115200 << Value);
                                        else ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(4800 << Value);
                                break;
                                case ra_USER_RES_LEN:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 4;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->CommaText = "128,256,512,1024";
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(128 << Value);
                                break;
                                case ra_USER_LAS_INT:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 2;
                                        for (BYTE j = 0; j <= con_cLas_Int; j++)
                                                ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->Add(IntToStr(j));
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_EXT_PORT:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_EXT_DIV:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 5;
                                        for (WORD j = 0; j <= con_cExt_Div; j++)
                                                ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->Add(IntToStr(j + 1));
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value + 1);
                                break;
                                case ra_USER_RES_WAIT:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 5;
                                        for (WORD j = 0; j <= con_cLock_Time; j++)
                                                ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->Add(IntToStr(5 * j));
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(5 * Value);
                                break;
                                case ra_USER_WND_LW:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 3;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_WND_TH:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 3;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_PNT_TIME:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 5;
                                        for (WORD j = 0; j <= con_cPxl_Time; j++)
                                                ValueListEditor_User_Parameters->ItemProps[ind_Par]->PickList->Add(IntToStr(2 * j + 10));
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(2 * Value + 10);
                                break;
                                case ra_USER_EXPOSURE:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 5;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                                case ra_USER_EXPOS_DELAY:
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->MaxLength = 5;
                                        ValueListEditor_User_Parameters->ItemProps[ind_Par]->ReadOnly = true;
                                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                                break;
                        }
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::View_Factory_Parameters()
{
        BYTE ind_Par;
        WORD Value;
        if (ValueListEditor_Factory_Parameters->RowCount == con_FACTORY_REGISTERS + 1) {
                for (ind_Par = 0; ind_Par < con_FACTORY_REGISTERS; ind_Par++) {
                        Value = FACTORY_PARAMETERS.REG_ADDR[ind_Par];
                        ValueListEditor_Factory_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                }
        } else {
                for (ind_Par = 0; ind_Par < con_FACTORY_REGISTERS; ind_Par++) {
                        Value = FACTORY_PARAMETERS.REG_ADDR[ind_Par];
                        ValueListEditor_Factory_Parameters->InsertRow(str_FACTORY_PARAMETERS_NAME[ind_Par], IntToStr(Value), true);
                        ValueListEditor_Factory_Parameters->ItemProps[ind_Par]->EditStyle = es_FACTORY_PARAMETERS[ind_Par];
                        ValueListEditor_Factory_Parameters->ItemProps[ind_Par]->MaxLength = 5;
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Update_User_Parameters()
{
        BYTE ind_Par = ind_User_Param;
        WORD Value;

        switch (ind_Par){
                case ra_USER_POWER:
                        if (ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] == "on") Value = 1;
                        else Value = 0;
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_ADDR:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_RSBR:
                        switch (StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1])) {
                                case 4800:
                                        Value = 0x40;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 9600:
                                        Value = 0x41;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 19200:
                                        Value = 0x42;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 38400:
                                        Value = 0x43;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 115200:
                                        Value = 0x00;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 230400:
                                        Value = 0x01;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 460800:
                                        Value = 0x02;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 921600:
                                        Value = 0x03;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                        }
                break;
                case ra_USER_RES_LEN:
                        switch (StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1])) {
                                case 128:
                                        Value = 0;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 256:
                                        Value = 1;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 512:
                                        Value = 2;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                                case 1024:
                                        Value = 3;
                                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                                break;
                        }
                break;
                case ra_USER_LAS_INT:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_EXT_PORT:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_EXT_DIV:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value - 1;
                break;
                case ra_USER_RES_WAIT:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value / 5;
                break;
                case ra_USER_WND_LW:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        Value = USER_PARAMETERS.REG_ADDR[ind_Par];
                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_WND_TH:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        Value = USER_PARAMETERS.REG_ADDR[ind_Par];
                        ValueListEditor_User_Parameters->Cells[1][ind_Par + 1] = IntToStr(Value);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_PNT_TIME:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = (Value - 10) / 2;
                break;
                case ra_USER_EXPOSURE:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
                case ra_USER_EXPOS_DELAY:
                        Value = StrToInt(ValueListEditor_User_Parameters->Cells[1][ind_Par + 1]);
                        USER_PARAMETERS.REG_ADDR[ind_Par] = Value;
                break;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Update_Factory_Parameters()
{
        BYTE ind_Par = ind_Factory_Param;
        WORD Value;
        Value = StrToInt(ValueListEditor_Factory_Parameters->Cells[1][ind_Par + 1]);
        FACTORY_PARAMETERS.REG_ADDR[ind_Par] = Value;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_ConnectClick(TObject *Sender)
{
        if (Button_Connect->Caption == "Connect") {
                RF620INT_Parameters.Mode = INT_USB_RS;
                RF620INT_Parameters.lpszName = "D2XX Recovery PID for XP";
                RF620INT_Parameters.BaudRate = StrToInt(ComboBox_Connect_Baud->Text);
                RF620INT_Parameters.NET_Address = StrToInt(ComboBox_Connect_NetAddress->Text);
                ROI_Window.Left = PaintBox_BitMap->Left;
                ROI_Window.Top = PaintBox_BitMap->Top;
                ROI_Window.Right = PaintBox_BitMap->Width;
                ROI_Window.Bottom = PaintBox_BitMap->Height;
/*                Set_ROI = false;
                RF620_status = RF620Interface->Connect(&RF620INT_Parameters);
                if (RF620_status != RF620INT_OK) return;
                RF620_status = RF620Interface->RF620_Hello(&HELLO_PARAMETERS);
                if (RF620_status != RF620INT_OK) return;
                RF620_status = RF620Interface->RF620_Get_Factory_Params(&FACTORY_PARAMETERS);
                if (RF620_status != RF620INT_OK) return;
                RF620_status = RF620Interface->RF620_Get_User_Params(&USER_PARAMETERS);
                if (RF620_status != RF620INT_OK) return;
                RF620_status = RF620Interface->RF620_Get_Angle(&fAngle);
                if (RF620_status != RF620INT_OK) return;
                Panel_Angle->Caption = "Angle = "+ FloatToStrF(fAngle * 180 / M_PI, ffFixed, 3, 3) + "";
                View_Factory_Parameters();
                View_User_Parameters();*/
                Label_Ranges_Model_Value->Caption = IntToStr(HELLO_PARAMETERS.REG_NAME.DEVICE_TYPE);
                Label_Ranges_Serial_Value->Caption = IntToStr(HELLO_PARAMETERS.REG_NAME.DEVICE_SERIAL_HI << 16 | HELLO_PARAMETERS.REG_NAME.DEVICE_SERIAL_LO);
                Label_Ranges_Base_Value->Caption = IntToStr(HELLO_PARAMETERS.REG_NAME.BASE_Z = 150) + " mm";
                Label_Ranges_RangeZ_Value->Caption = IntToStr(HELLO_PARAMETERS.REG_NAME.RANGE_Z = 250) + " mm";
                Label_Ranges_RangeXs_Value->Caption = IntToStr(HELLO_PARAMETERS.REG_NAME.RANGE_XS = 115) + " mm";
                Label_Ranges_RangeXe_Value->Caption = IntToStr(HELLO_PARAMETERS.REG_NAME.RANGE_XE = 230) + " mm";
                Chart_Result_Oscilloscope->LeftAxis->Minimum = 0;
                Chart_Result_Oscilloscope->LeftAxis->Maximum = HELLO_PARAMETERS.REG_NAME.RANGE_Z + HELLO_PARAMETERS.REG_NAME.RANGE_Z * 0.05;
                Chart_Result_Oscilloscope->BottomAxis->Minimum = -((HELLO_PARAMETERS.REG_NAME.RANGE_XE +  HELLO_PARAMETERS.REG_NAME.RANGE_XE * 0.05) / 2);
                Chart_Result_Oscilloscope->BottomAxis->Maximum = (HELLO_PARAMETERS.REG_NAME.RANGE_XE +  HELLO_PARAMETERS.REG_NAME.RANGE_XE  * 0.05) / 2;

                Button_Connect->Caption = "Disconnect";
                ComboBox_Connect_Baud->Enabled = false;
                ComboBox_Connect_NetAddress->Enabled = false;
                Button_Get_Result->Enabled = true;
                Timer_Resize->Enabled = true;
                PageControl_Interface->Enabled = true;
                PageControl_Interface->ActivePage = TabSheet_Result;
                ROI_Window.Left = (USER_PARAMETERS.REG_NAME.WINDOW_LW.BITs.LEFT * con_BMP_Width / 100);
                ROI_Window.Top = (USER_PARAMETERS.REG_NAME.WINDOW_TH.BITs.TOP * con_BMP_Height / 100);
                ROI_Window.Right = ROI_Window.Left + (USER_PARAMETERS.REG_NAME.WINDOW_LW.BITs.WIDTH * con_BMP_Width / 100);
                ROI_Window.Bottom = ROI_Window.Top + (USER_PARAMETERS.REG_NAME.WINDOW_TH.BITs.HEIGHT * con_BMP_Height / 100);
                if ((ROI_Window.Width() != 0) && (ROI_Window.Height() != 0))
                        FrameFreq = ((float) 1000 / (float) (0.000025 * (ROI_Window.Width() + 152) * ROI_Window.Height() + 0.1) + 0.5);
                Panel_Freq->Caption = "Freq.:" + IntToStr(FrameFreq) + " fps";
        } else {
                RF620_status = RF620Interface->Disonnect();
                Label_Ranges_Model_Value->Caption = "none";
                Label_Ranges_Serial_Value->Caption = "none";
                Label_Ranges_Base_Value->Caption = "none mm";
                Label_Ranges_RangeZ_Value->Caption = "none mm";
                Label_Ranges_RangeXs_Value->Caption = "none mm";
                Label_Ranges_RangeXe_Value->Caption = "none mm";
                Button_Connect->Caption = "Connect";
                ComboBox_Connect_Baud->Enabled = true;
                ComboBox_Connect_NetAddress->Enabled = true;
                Timer_Resize->Enabled = true;
                Button_Get_Result->Enabled = false;
                Button_Get_Video->Enabled = false;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_User_ParametersContextPopup(
      TObject *Sender, TPoint &MousePos, bool &Handled)
{
        Update_User_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_User_ParametersEditButtonClick(
      TObject *Sender)
{
        switch(ind_User_Param) {
                case ra_USER_EXPOSURE:
                        Form_Exposure->Please_Update = false;
                        Form_Exposure->CheckBox_bit_Auto_Exposure->Checked = USER_PARAMETERS.REG_NAME.EXPOSURE.BITs.AUTO;
                        Form_Exposure->Edit_Exposure_Value->Text = IntToStr(10 * USER_PARAMETERS.REG_NAME.EXPOSURE.BITs.VALUE + 50);
                        Form_Exposure->ShowModal();
                        if (Form_Exposure->Please_Update) {
                                if (Form_Exposure->CheckBox_bit_Auto_Exposure->Checked)
                                        USER_PARAMETERS.REG_NAME.EXPOSURE.BITs.AUTO = 1;
                                else USER_PARAMETERS.REG_NAME.EXPOSURE.BITs.AUTO = 0;
                                USER_PARAMETERS.REG_NAME.EXPOSURE.BITs.VALUE = ((StrToInt(Form_Exposure->Edit_Exposure_Value->Text) - 50 + 5) / 10);
                        }
                break;
                case ra_USER_EXPOS_DELAY:
                        Form_Exposure_Delay->Please_Update = false;
                        Form_Exposure_Delay->Edit_Exposure_Value->Text = IntToStr(5 * USER_PARAMETERS.REG_NAME.EXPOSURE_DELAY);
                        Form_Exposure_Delay->ShowModal();
                        if (Form_Exposure_Delay->Please_Update) {
                                USER_PARAMETERS.REG_NAME.EXPOSURE_DELAY = ((StrToInt(Form_Exposure_Delay->Edit_Exposure_Value->Text) + 3) / 5);
                        }
                break;
                case ra_USER_EXT_PORT:
                        Form_Ext_Sig->Please_Update = false;
                        Form_Ext_Sig->RadioGroupPriority->ItemIndex = USER_PARAMETERS.REG_NAME.EXTERNAL_PORTS.BITs.OUT_PRIORITY;
                        Form_Ext_Sig->RadioGroupStrobeLevel->ItemIndex = USER_PARAMETERS.REG_NAME.EXTERNAL_PORTS.BITs.STROBE_ACTIV;
                        Form_Ext_Sig->RadioGroupExternalSync->ItemIndex = USER_PARAMETERS.REG_NAME.EXTERNAL_PORTS.BITs.EXTERNAL_SINC;
                        Form_Ext_Sig->ShowModal();
                        if (Form_Ext_Sig->Please_Update) {
                                USER_PARAMETERS.REG_NAME.EXTERNAL_PORTS.BITs.OUT_PRIORITY = Form_Ext_Sig->RadioGroupPriority->ItemIndex ;
                                USER_PARAMETERS.REG_NAME.EXTERNAL_PORTS.BITs.STROBE_ACTIV = Form_Ext_Sig->RadioGroupStrobeLevel->ItemIndex;
                                USER_PARAMETERS.REG_NAME.EXTERNAL_PORTS.BITs.EXTERNAL_SINC = Form_Ext_Sig->RadioGroupExternalSync->ItemIndex;
                        }
                break;
        }
        View_User_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_User_ParametersExit(
      TObject *Sender)
{
        Update_User_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_User_ParametersKeyPress(
      TObject *Sender, char &Key)
{
        Update_User_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_User_ParametersSelectCell(
      TObject *Sender, int ACol, int ARow, bool &CanSelect)
{
        Update_User_Parameters();
        ind_User_Param = ARow - 1;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_Factory_ParametersContextPopup(
      TObject *Sender, TPoint &MousePos, bool &Handled)
{
        Update_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_Factory_ParametersEditButtonClick(
      TObject *Sender)
{
        Update_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_Factory_ParametersExit(
      TObject *Sender)
{
        Update_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_Factory_ParametersKeyPress(
      TObject *Sender, char &Key)
{
        Update_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::ValueListEditor_Factory_ParametersSelectCell(
      TObject *Sender, int ACol, int ARow, bool &CanSelect)
{
        Update_Factory_Parameters();
        ind_Factory_Param = ARow - 1;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Read_User_ParameterClick(TObject *Sender)
{
        WORD Value = USER_PARAMETERS.REG_ADDR[ind_User_Param];
        RF620_status = RF620Interface->RF620_Get_User_Param(ind_User_Param, &Value);
        if (RF620_status != RF620INT_OK) return;
        USER_PARAMETERS.REG_ADDR[ind_User_Param] = Value;
        View_User_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Load_User_ParameterClick(TObject *Sender)
{
        WORD Value = USER_PARAMETERS.REG_ADDR[ind_User_Param];
        RF620_status = RF620Interface->RF620_Set_User_Param(ind_User_Param, &Value);
        if (RF620_status != RF620INT_OK) return;
        View_User_Parameters();
        ComboBox_Connect_NetAddress->ItemIndex = USER_PARAMETERS.REG_ADDR[ra_USER_ADDR];
        switch (USER_PARAMETERS.REG_ADDR[ra_USER_RSBR]) {
                case 0x40:
                        ComboBox_Connect_Baud->ItemIndex = 0;
                break;
                case 0x41:
                        ComboBox_Connect_Baud->ItemIndex = 1;
                break;
                case 0x42:
                        ComboBox_Connect_Baud->ItemIndex = 2;
                break;
                case 0x43:
                        ComboBox_Connect_Baud->ItemIndex = 3;
                break;
                case 0x00:
                        ComboBox_Connect_Baud->ItemIndex = 4;
                break;
                case 0x01:
                        ComboBox_Connect_Baud->ItemIndex = 5;
                break;
                case 0x02:
                        ComboBox_Connect_Baud->ItemIndex = 6;
                break;
                case 0x03:
                        ComboBox_Connect_Baud->ItemIndex = 7;
                break;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Read_User_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Get_User_Params(&USER_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_User_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Load_User_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Set_User_Params(&USER_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_User_Parameters();
        ComboBox_Connect_NetAddress->ItemIndex = USER_PARAMETERS.REG_ADDR[ra_USER_ADDR];
        switch (USER_PARAMETERS.REG_ADDR[ra_USER_RSBR]) {
                case 0x40:
                        ComboBox_Connect_Baud->ItemIndex = 0;
                break;
                case 0x41:
                        ComboBox_Connect_Baud->ItemIndex = 1;
                break;
                case 0x42:
                        ComboBox_Connect_Baud->ItemIndex = 2;
                break;
                case 0x43:
                        ComboBox_Connect_Baud->ItemIndex = 3;
                break;
                case 0x00:
                        ComboBox_Connect_Baud->ItemIndex = 4;
                break;
                case 0x01:
                        ComboBox_Connect_Baud->ItemIndex = 5;
                break;
                case 0x02:
                        ComboBox_Connect_Baud->ItemIndex = 6;
                break;
                case 0x03:
                        ComboBox_Connect_Baud->ItemIndex = 7;
                break;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Write_User_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_User_Params(FALSE);
        if (RF620_status != RF620INT_OK) return;
        RF620_status = RF620Interface->RF620_Get_User_Params(&USER_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_User_Parameters();
        ComboBox_Connect_NetAddress->ItemIndex = USER_PARAMETERS.REG_ADDR[ra_USER_ADDR];
        switch (USER_PARAMETERS.REG_ADDR[ra_USER_RSBR]) {
                case 0x40:
                        ComboBox_Connect_Baud->ItemIndex = 0;
                break;
                case 0x41:
                        ComboBox_Connect_Baud->ItemIndex = 1;
                break;
                case 0x42:
                        ComboBox_Connect_Baud->ItemIndex = 2;
                break;
                case 0x43:
                        ComboBox_Connect_Baud->ItemIndex = 3;
                break;
                case 0x00:
                        ComboBox_Connect_Baud->ItemIndex = 4;
                break;
                case 0x01:
                        ComboBox_Connect_Baud->ItemIndex = 5;
                break;
                case 0x02:
                        ComboBox_Connect_Baud->ItemIndex = 6;
                break;
                case 0x03:
                        ComboBox_Connect_Baud->ItemIndex = 7;
                break;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Restore_User_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_User_Params(TRUE);
        if (RF620_status != RF620INT_OK) return;
        RF620_status = RF620Interface->RF620_Get_User_Params(&USER_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_User_Parameters();
        ComboBox_Connect_NetAddress->ItemIndex = USER_PARAMETERS.REG_ADDR[ra_USER_ADDR];
        switch (USER_PARAMETERS.REG_ADDR[ra_USER_RSBR]) {
                case 0x40:
                        ComboBox_Connect_Baud->ItemIndex = 0;
                break;
                case 0x41:
                        ComboBox_Connect_Baud->ItemIndex = 1;
                break;
                case 0x42:
                        ComboBox_Connect_Baud->ItemIndex = 2;
                break;
                case 0x43:
                        ComboBox_Connect_Baud->ItemIndex = 3;
                break;
                case 0x00:
                        ComboBox_Connect_Baud->ItemIndex = 4;
                break;
                case 0x01:
                        ComboBox_Connect_Baud->ItemIndex = 5;
                break;
                case 0x02:
                        ComboBox_Connect_Baud->ItemIndex = 6;
                break;
                case 0x03:
                        ComboBox_Connect_Baud->ItemIndex = 7;
                break;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Rotate_SystemClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Rotate_Angle(&fAngle);
        if (RF620_status != RF620INT_OK) return;
        Panel_Angle->Caption = "Angle = "+ FloatToStrF(fAngle * 180 / M_PI, ffFixed, 3, 3) + "";
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Write_AngleClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_Angle(FALSE);
        if (RF620_status != RF620INT_OK) return;
        RF620_status = RF620Interface->RF620_Get_Angle(&fAngle);
        if (RF620_status != RF620INT_OK) return;
        Panel_Angle->Caption = "Angle = "+ FloatToStrF(fAngle * 180 / M_PI, ffFixed, 3, 3) + "";
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Restore_AngleClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_Angle(TRUE);
        if (RF620_status != RF620INT_OK) return;
        RF620_status = RF620Interface->RF620_Get_Angle(&fAngle);
        if (RF620_status != RF620INT_OK) return;
        Panel_Angle->Caption = "Angle = "+ FloatToStrF(fAngle * 180 / M_PI, ffFixed, 3, 3) + "";
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Read_Factory_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Get_Factory_Params(&FACTORY_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Load_Factory_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Set_Factory_Params(&FACTORY_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Write_Factory_ParametersClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_Factory_Params(FALSE);
        if (RF620_status != RF620INT_OK) return;
        RF620_status = RF620Interface->RF620_Get_Factory_Params(&FACTORY_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Reastore_Factory_ParametersClick(
      TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_Factory_Params(TRUE);
        if (RF620_status != RF620INT_OK) return;
        RF620_status = RF620Interface->RF620_Get_Factory_Params(&FACTORY_PARAMETERS);
        if (RF620_status != RF620INT_OK) return;
        View_Factory_Parameters();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Timer_ResizeTimer(TObject *Sender)
{
        if (Button_Connect->Caption == "Disconnect") {
                if ((Form_Main->Constraints->MinHeight + 50) <= Form_Main->Constraints->MaxHeight)
                        Form_Main->Constraints->MinHeight = Form_Main->Constraints->MinHeight + 50;
                else {
                        Form_Main->Height = Form_Main->Constraints->MaxHeight;
                        Form_Main->Constraints->MinHeight = Form_Main->Constraints->MaxHeight;
                        Timer_Resize->Enabled = false;
                }
        } else {
                if ((Form_Main->Constraints->MinHeight - 50) >= Min_Height) {
                        Form_Main->Constraints->MinHeight = Form_Main->Constraints->MinHeight - 50;
                        Form_Main->Height = Form_Main->Height - 50;
                } else {
                        Form_Main->Constraints->MinHeight = Min_Height;
                        Form_Main->Height = Min_Height;
                        Timer_Resize->Enabled = false;
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Get_ResultClick(TObject *Sender)
{
        if (Button_Get_Result->Caption == "Start Measure") {
                Button_Get_Result->Caption = "Stop Measure";
                Timer_Result->Enabled = true;
                PageControl_Interface->Enabled = false;
        } else {
                Button_Get_Result->Caption = "Start Measure";
                Timer_Result->Enabled = false;
                PageControl_Interface->Enabled = true;
        }
/*        BYTE nPack;
        WORD nPoint;
        DWORD TimeZero = GetTickCount();
        DWORD Counter = 0;
        Randomize();
while ((GetTickCount() - TimeZero) < 10000) {
        RF620_status = RF620Interface->RF620_Get_Line_Result(&nPack, &nPoint, &RESULT_BUFFER[0]);
//        if (RF620_status != RF620INT_OK) return;
        Counter ++;
        Caption = " = " + IntToStr(Counter);
}
        Caption = " = " + IntToStr(Counter) + "     = " + FloatToStrF((float)((float)Counter / (float)10), ffFixed, 6, 3);
  */
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Timer_ResultTimer(TObject *Sender)
{
        BYTE nPack;
        WORD nPoint;
        AnsiString str = "";
        DWORD TimeZero = GetTickCount();
        RF620_status = RF620Interface->RF620_Get_Line_Result(&nPack, &nPoint, &RESULT_BUFFER[0]);
        Caption = IntToStr(nPack);
        if (RF620_status != RF620INT_OK) return;/*{
                switch (RF620_status)    {
                case RF620INT_PORT_NOT_FOUND:
                        str = "Port not found.";
                break;
                case RF620INT_PORT_NOT_OPENNED:
                        str = "Port not openned.";
                break;
                case RF620INT_READ_TIMEOUT:
                        str = "Read timeout.";
                break;
                case RF620INT_READ_ERROR:
                        str = "Read error.";
                break;
                case RF620INT_WRITE_TIMEOUT:
                        str = "Write timeout.";
                break;
                case RF620INT_WRITE_ERROR:
                        str = "Write error.";
                break;
                case RF620INT_OTHER_ERROR:
                        str = "Unknown error.";
                break;
                case RF620INT_PACKET_ERROR:
                        str = "Packet error.";
                break;
                case RF620INT_NOT_CONNECTED:
                        str = "Connection close.";
                break;
                case RF620INT_FLASH_ERROR:
                        str = "FLASH error.";
                break;
                }
                RF620_status = RF620INT_OK;
                Application->MessageBox(str.c_str(), "RF620-SP.", MB_ICONERROR);

        } else { */
        if (PopupMenu_Save_Data->Items->Items[3]->Checked == false)
                Series_Result_Oscilloscope->Clear();
        DATA_TO_FILE_Pnt = nPoint;
        for (WORD ind = 0; ind < 1024; ind ++) {
                DATA_TO_FILE[ind][0] = DATA_TO_FILE[ind][1] = 0;
        }
        for (WORD ind = 1; ind < nPoint; ind ++) {
                short int ValueZ = RESULT_BUFFER[2 * ind];
                short int ValueX = RESULT_BUFFER[2 * ind + 1];
                if ((ValueZ & 0x2000) == 0x2000) ValueZ |= 0xC000;
                if ((ValueX & 0x2000) == 0x2000) ValueX |= 0xC000;
                if ((ValueZ != (short int)0xE000) && (ValueX != (short int)0xE000)) {
                        float X = (float)(ValueX * HELLO_PARAMETERS.REG_NAME.RANGE_XE) / (float)0x1000;
                        float Z = (float)(ValueZ * HELLO_PARAMETERS.REG_NAME.RANGE_Z) / (float)0x1000;
                        DATA_TO_FILE[ind - 1][0] = X;
                        DATA_TO_FILE[ind - 1][1] = Z;
                        Series_Result_Oscilloscope->AddXY(X, Z, FloatToStrF(X, ffFixed, 5, 2), clGreen);
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Get_VideoClick(TObject *Sender)
{
/*        WORD Bmp_Left, Bmp_Top, Bmp_Width, Bmp_Height;
        RF620_status = RF620Interface->RF620_Get_BitMap(&BITMAP_BUFFER[0], &Bmp_Left, &Bmp_Top, &Bmp_Width, &Bmp_Height);
        if (RF620_status != RF620INT_OK) return;
        PaintBox_BitMap->Left = Bmp_Left;
        PaintBox_BitMap->Top = Bmp_Top;
        PaintBox_BitMap->Width = Bmp_Width;
        PaintBox_BitMap->Height = Bmp_Height;
        pBitMap->Width = Bmp_Width;
        pBitMap->Height = Bmp_Height;
        for (WORD y = 0; y < pBitMap->Height; y++) {
                BYTE *ptr_In = (BYTE*) pBitMap->ScanLine[y];
                BYTE *ptr_Out = &BITMAP_BUFFER[y * Bmp_Width];
                for (WORD x = 0; x < pBitMap->Width; x++)
                ptr_In[x] = ptr_Out[x] << 1;
        }*/
        if (Button_Get_Video->Caption == "Get BITMAP") {
                Button_Get_Video->Caption = "Stop";
                Timer_Video->Enabled = true;
        } else {
                Button_Get_Video->Caption = "Get BITMAP";
                Timer_Video->Enabled = false;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_TABLE_TransClick(TObject *Sender)
{
        for (int j = 0; j < 2 * con_TBL_Height; j ++) {
                for (int i = 0; i < con_TBL_Width; i ++) {
                        if (j < con_TBL_Height) CALIBR_BUFFER_TABLES[j * con_TBL_Width + i] = (short int)(CALIBR_BUFFER_Z[i][j * 10]);
                        else CALIBR_BUFFER_TABLES[j * con_TBL_Width + i] = (short int)(CALIBR_BUFFER_X[i][(j - con_TBL_Height) * 10]);
                }
        }
        RF620_status = RF620Interface->RF620_Set_Factory_Tables(&CALIBR_BUFFER_TABLES[0], 2 * con_TBL_Width * con_TBL_Height);
        if (RF620_status != RF620INT_OK) return;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_TABLE_SaveClick(TObject *Sender)
{
        RF620_status = RF620Interface->RF620_Save_Factory_Tables();
        if (RF620_status != RF620INT_OK) return;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Timer_RepaintTimer(TObject *Sender)
{
        WORD FrameFreq;
        Timer_Repaint->Enabled = false;
        PaintBox_BitMap->Canvas->Draw(0,0, pBitMap);
        PaintBox_BitMap->Canvas->Pen->Color = clWhite;
        PaintBox_BitMap->Canvas->Pen->Width = 1;
        PaintBox_BitMap->Canvas->Pen->Style = psDot;
        PaintBox_BitMap->Canvas->Brush->Color = clBlack;
        PaintBox_BitMap->Canvas->Brush->Style = bsClear;
        PaintBox_BitMap->Canvas->Rectangle(ROI_Window);
        Timer_Repaint->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Motor_ConnectClick(TObject *Sender)
{
        if (Button_Motor_Connect->Caption == "Connect") {
                MOTORINT_Parameters.Mode = INT_USB_RS;
                MOTORINT_Parameters.lpszName = "RIFTEK MOTOR DRIVER INTERFACE";
                MOTORINT_Parameters.BaudRate = 230400;
                MOTOR_status = MOTORInterface->Connect(&MOTORINT_Parameters);
                if (MOTOR_status != MOTORINT_OK) return;
                Button_Motor_Connect->Caption = "Disconnect";
        } else {
                MOTOR_status = MOTORInterface->Disonnect();
                Button_Motor_Connect->Caption = "Connect";
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Motor_InitClick(TObject *Sender)
{
        BYTE Motor_SysW;
        MOTOR_status = MOTORInterface->GetSys(&Motor_SysW);
        if (MOTOR_status != MOTORINT_OK) return;
        if (!(Motor_SysW & b_MT_Pos_B))  {
                MOTOR_status = MOTORInterface->MotStop(&Motor_SysW);
                if (MOTOR_status != MOTORINT_OK) return;
                MOTOR_status = MOTORInterface->MotInit();
                if (MOTOR_status != MOTORINT_OK) return;
                Timer_Motor->Tag = 0;
                Timer_Motor->Enabled = true;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Motor_ForwardClick(TObject *Sender)
{
        BYTE Motor_SysW;
        DWORD Value = SpinEdit_Motor_Move_Steps->Value;
        MOTOR_status = MOTORInterface->MotStop(&Motor_SysW);
        if (MOTOR_status != MOTORINT_OK) return;
        MOTOR_status = MOTORInterface->MotGoFw(Value);
        if (MOTOR_status != MOTORINT_OK) return;
        Timer_Motor->Tag = 0;
        Timer_Motor->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Motor_BackwardClick(TObject *Sender)
{
        BYTE Motor_SysW;
        DWORD Value = SpinEdit_Motor_Move_Steps->Value;
        MOTOR_status = MOTORInterface->GetSys(&Motor_SysW);
        if (MOTOR_status != MOTORINT_OK) return;
        if (!(Motor_SysW & b_MT_Pos_B)) {
                MOTOR_status = MOTORInterface->MotGoBw(Value);
                if (MOTOR_status != MOTORINT_OK) return;
                Timer_Motor->Tag = 0;
                Timer_Motor->Enabled = true;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_CalibrationClick(TObject *Sender)
{
        BYTE Motor_SysW;
        DWORD Value = SpinEdit_Motor_Move_Start->Value / con_MOTOR_step_um;
        MOTOR_status = MOTORInterface->MotStop(&Motor_SysW);
        if (MOTOR_status != MOTORINT_OK) return;
        MOTOR_status = MOTORInterface->MotGoFw(Value);
        if (MOTOR_status != MOTORINT_OK) return;
        Timer_Motor->Tag = 1;
        Timer_Motor->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Timer_MotorTimer(TObject *Sender)
{
        BYTE Motor_SysW;
        DWORD delta_Steps;
        WORD nData, ind_Data;
        BYTE nMeasure = 3;
        if (Timer_Motor->Tag == 0) {
                Timer_Motor->Enabled = false;
                MOTOR_status = MOTORInterface->GetSys(&Motor_SysW);
                if (MOTOR_status != MOTORINT_OK) return;
                if (!(Motor_SysW & (b_MT_Mv_Fw | b_MT_Mv_Bw))) {
                        MOTOR_status = MOTORInterface->MotStop(&Motor_SysW);
                        if (MOTOR_status != MOTORINT_OK) return;
                } else Timer_Motor->Enabled = true;
        } else {
                Timer_Motor->Enabled = false;
                MOTOR_status = MOTORInterface->GetSys(&Motor_SysW);
                if (MOTOR_status != MOTORINT_OK) return;
                switch (Timer_Motor->Tag) {
                        case 1:
                                if (!(Motor_SysW & (b_MT_Mv_Fw | b_MT_Mv_Bw))) {
                                        delta_Um = (float)((float)HELLO_PARAMETERS.REG_NAME.RANGE_Z * 1000 + (float)HELLO_PARAMETERS.REG_NAME.RANGE_Z * 20) / (float)con_DIAP_Z_Steps;
                                        delta_Um = floor((delta_Um + con_MOTOR_step_um / 2) / con_MOTOR_step_um);
                                        delta_Um *= con_MOTOR_step_um;
                                        Timer_Motor->Tag = 2;
                                        ind_Step = 0;
                                        nSteps = con_DIAP_Z_Steps;
                                }
                                Timer_Motor->Enabled = true;
                        break;
                        case 2:
                                if (!(Motor_SysW & (b_MT_Mv_Fw | b_MT_Mv_Bw))) {
                                        if (ind_Step < nSteps) {
                                                for (ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++) {
                                                        CALIBR_BUFFER[ind_Data][ind_Step] = 0;
                                                        CALIBR_BUFFER_NMEAS[ind_Data] = 0;
                                                }
                                                for (BYTE ind_Measure = 0; ind_Measure < nMeasure; ind_Measure ++) {
                                                        RF620_status = RF620Interface->RF620_Get_NO_Line_Result(&RESULT_BUFFER[0]);
                                                        if (RF620_status != RF620INT_OK) return;
                                                        nData = RESULT_BUFFER[0];
                                                        WORD ValueX, ValueY;
                                                        for (ind_Data = 0; ind_Data < nData; ind_Data ++) {
                                                                ValueX = ind_Data;
                                                                ValueY = RESULT_BUFFER[ind_Data + 1];
                                                                if ((ValueX) && (ValueY)) {
                                                                        CALIBR_BUFFER[ValueX][ind_Step] += ValueY;
                                                                        CALIBR_BUFFER_NMEAS[ValueX] ++;
                                                                }
                                                        }
                                                }
                                                Series_Calibr_Oscilloscope->Clear();
                                                for (ind_Data = 0; ind_Data < nData; ind_Data ++) {
                                                        if (CALIBR_BUFFER_NMEAS[ind_Data] == nMeasure)
                                                                CALIBR_BUFFER[ind_Data][ind_Step] = (CALIBR_BUFFER[ind_Data][ind_Step] + CALIBR_BUFFER_NMEAS[ind_Data] / 2) / CALIBR_BUFFER_NMEAS[ind_Data];
                                                        else CALIBR_BUFFER[ind_Data][ind_Step] = 0;
                                                        Series_Calibr_Oscilloscope->AddXY(ind_Data, CALIBR_BUFFER[ind_Data][ind_Step], IntToStr(ind_Data), clRed);
                                                }
                                        }
                                        ind_Step ++;
                                        delta_Steps = (delta_Um * ind_Step - delta_Um * (ind_Step - 1)) / con_MOTOR_step_um;
                                }
                                if (ind_Step == nSteps) {
                                        MOTOR_status = MOTORInterface->MotStop(&Motor_SysW);
                                        if (MOTOR_status != MOTORINT_OK) return;
                                        Timer_Motor->Enabled = false;
                                } else {
                                        MOTOR_status = MOTORInterface->MotGoFw(delta_Steps);
                                        if (MOTOR_status != MOTORINT_OK) return;
                                        Timer_Motor->Enabled = true;
                                }
                        break;
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::TabSheet_ResultShow(TObject *Sender)
{
        if (Button_Connect->Caption != "Connect") {
                Button_Get_Result->Enabled = true;
                Button_Get_Video->Enabled = false;
                Timer_Repaint->Enabled = false;
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm_Main::TabSheet_VideoShow(TObject *Sender)
{
        if (Button_Connect->Caption != "Connect") {
                Button_Get_Result->Enabled = false;
                Button_Get_Video->Enabled = true;
                Timer_Repaint->Enabled = true;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::TabSheet_CalibrationShow(TObject *Sender)
{
        Button_Get_Result->Enabled = false;
        Button_Get_Video->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Calibr_Save_FileClick(TObject *Sender)
{
        int iFileHandle;
        int iFileLength;
        if (!DirectoryExists(CurrDirectory + "\\DataFiles"))
                CreateDir(CurrDirectory + "\\DataFiles");
        SaveDialogData->InitialDir = CurrDirectory + "\\DataFiles";
        SaveDialogData->Filter = "All files(*.*)|*.*|Data files(*.dtf)|*.dtf";
        SaveDialogData->FilterIndex = 2;
        SaveDialogData->DefaultExt = "dtf";
        if (SaveDialogData->Execute()) {
                iFileHandle = FileCreate(SaveDialogData->FileName);
                FileWrite(iFileHandle, CALIBR_BUFFER, sizeof(CALIBR_BUFFER));
                FileClose(iFileHandle);
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Calibr_Load_FileClick(TObject *Sender)
{
        int iFileHandle;
        int iFileLength;
        if (!DirectoryExists(CurrDirectory + "\\DataFiles"))
                CreateDir(CurrDirectory + "\\DataFiles");
        OpenDialogData->InitialDir = CurrDirectory + "\\DataFiles";
        OpenDialogData->Filter = "All files(*.*)|*.*|Data files(*.dtf)|*.dtf";
        OpenDialogData->FilterIndex = 2;
        OpenDialogData->DefaultExt = "dtf";
        if (OpenDialogData->Execute()) {
                iFileHandle = FileOpen(OpenDialogData->FileName, fmOpenRead);
                FileRead(iFileHandle, CALIBR_BUFFER, sizeof(CALIBR_BUFFER));
                FileClose(iFileHandle);
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Convert_Line_Pixel()
{
        for (ind_Step = 0; ind_Step < con_DIAP_Z_Steps; ind_Step ++)
                for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++)
                        CALIBR_BUFFER_PXL[ind_Data][ind_Step] = 0;

        for (ind_Step = 0; ind_Step < con_DIAP_Z_Steps - 0; ind_Step ++) {
                WORD ind_Data;
                WORD X_Right = StrToInt(Edit_X_Right->Text),
                     X_Left = StrToInt(Edit_X_Left->Text);
                WORD BegX, Len;
                WORD min, max;
                WORD indMin, indMax, indSred;
                DWORD Sred;
                WORD XPos;
                WORD Value;
                Sred = 0;
                indSred = 0;
                ind_Data = 0;
                while (ind_Data < 150) {
                        while ((ind_Data < 150) && (CALIBR_BUFFER[ind_Data][ind_Step] == 0)) ind_Data++;
                        if (ind_Data >= 150) break;
                        BegX = ind_Data;
                        while ((ind_Data < 150) && (CALIBR_BUFFER[ind_Data][ind_Step] != 0)) {
                                Sred += CALIBR_BUFFER[ind_Data][ind_Step];
                                indSred ++;
                                ind_Data++;

                        }
                }
                ind_Data = con_TBL_Width - 150;
                while (ind_Data < con_TBL_Width) {
                        while ((ind_Data < con_TBL_Width) && (CALIBR_BUFFER[ind_Data][ind_Step] == 0)) ind_Data++;
                        if (ind_Data >= con_TBL_Width) break;
                        BegX = ind_Data;
                        while ((ind_Data < con_TBL_Width) && (CALIBR_BUFFER[ind_Data][ind_Step] != 0)) {
                                Sred += CALIBR_BUFFER[ind_Data][ind_Step];
                                indSred ++;
                                ind_Data++;

                        }
                }
                Sred /= indSred;
                ind_Data = 0;
                while (ind_Data < con_TBL_Width - X_Right) {
                        while ((ind_Data < con_TBL_Width - X_Right) && (CALIBR_BUFFER[ind_Data][ind_Step] == 0)) ind_Data++;
                        if (ind_Data >= con_TBL_Width - X_Right) break;
                        BegX = ind_Data;
                        while ((ind_Data < con_TBL_Width - X_Right) && (CALIBR_BUFFER[ind_Data][ind_Step] != 0)) {
                                if (abs(CALIBR_BUFFER[ind_Data][ind_Step] - Sred) > 180) {
                                        CALIBR_BUFFER[ind_Data][ind_Step] = 0;
                                }
                                ind_Data++;
                        }
                }
                ind_Data = X_Left;
                while (ind_Data < con_TBL_Width - X_Right) {
                        while ((ind_Data < con_TBL_Width - X_Right) && (CALIBR_BUFFER[ind_Data][ind_Step] == 0)) ind_Data++;
                        if (ind_Data >= con_TBL_Width - X_Right) break;
                        BegX = ind_Data;
                        min = 10000; max = 0;
                        while ((ind_Data < con_TBL_Width - X_Right) && (CALIBR_BUFFER[ind_Data][ind_Step] != 0)) {
                                if (CALIBR_BUFFER[ind_Data][ind_Step] > max) {
                                        max = CALIBR_BUFFER[ind_Data][ind_Step];
                                        indMax = ind_Data;
                                }
                                if (CALIBR_BUFFER[ind_Data][ind_Step] < min) {
                                        min = CALIBR_BUFFER[ind_Data][ind_Step];
                                        indMin = ind_Data;
                                }
                                ind_Data++;

                        }
                        if (ind_Data >= con_TBL_Width - X_Right) break;
                        Len = 0;
                        Sred = 0;
                        for (indSred = BegX; indSred < ind_Data; indSred ++) {
                                if ((ind_Data - BegX) > 2 ) {
                                        if ((indSred != indMax) && (indSred != indMin)) {
                                                Sred += CALIBR_BUFFER[indSred][ind_Step];
                                                Len ++;
                                        }
                                } else {
                                    Sred = CALIBR_BUFFER[BegX + (ind_Data - BegX + 1) / 2][ind_Step];
                                    Len = 1;
                                }
                        }
                        if (Len > 0) {
                                XPos = (BegX + (ind_Data - BegX + 1) / 2);
                                Value = (Sred + Len / 2) / Len;
                                CALIBR_BUFFER_PXL[XPos][ind_Step] = Value;
                        }
                }
        }
        Series_Calibr_Oscilloscope->Clear();
        for (ind_Step = 0; ind_Step < con_DIAP_Z_Steps; ind_Step ++)
                for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++)
                        if (CALIBR_BUFFER_PXL[ind_Data][ind_Step]) Series_Calibr_Oscilloscope->AddXY(ind_Data, CALIBR_BUFFER_PXL[ind_Data][ind_Step], IntToStr(ind_Data), clRed);

}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Convert_Pixel_Z()
{
        for (ind_Step = 0; ind_Step < 10 * con_TBL_Height; ind_Step ++)
                for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++)
                        CALIBR_BUFFER_Z[ind_Data][ind_Step] = (short int)0xE000;
        short int ValZ, OldValZ, iValZ, iOldValZ;
        delta_Um = (float)((float)HELLO_PARAMETERS.REG_NAME.RANGE_Z * 1000 + (float)HELLO_PARAMETERS.REG_NAME.RANGE_Z * 20) / (float)con_DIAP_Z_Steps;
        delta_Um = floor((delta_Um + con_MOTOR_step_um / 2) / con_MOTOR_step_um);
        delta_Um *= con_MOTOR_step_um;
        for (ind_Step = 0; ind_Step < con_DIAP_Z_Steps; ind_Step ++) {
                OldValZ = 0;
                for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++) {
                        ValZ = CALIBR_BUFFER_PXL[ind_Data][ind_Step];
                        iValZ = ind_Data;
                        if (ValZ != 0) {
                                if (OldValZ != 0) {
                                        float delta = (float)(ValZ - OldValZ) / (iValZ - iOldValZ);
                                        for (int k = 0; k <= (iValZ-iOldValZ); k++)
                                                CALIBR_BUFFER_Z[iOldValZ + k][OldValZ + (int)(k * delta)] = (int)((float)0x1000 * (float)ind_Step * (float)delta_Um) / ((float)HELLO_PARAMETERS.REG_NAME.RANGE_Z * (float)1000);
                                }
                                OldValZ = ValZ;
                                iOldValZ = iValZ;
                        }
                }
        }
        for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++) {
                OldValZ = (short int)0xE000;
                for (ind_Step = 0; ind_Step < 10 * con_TBL_Height; ind_Step ++) {
                        ValZ = CALIBR_BUFFER_Z[ind_Data][ind_Step];
                        iValZ = (short int)ind_Step;
                        if (ValZ != (short int)0xE000) {
                                if (OldValZ != (short int)0xE000) {
                                        float delta = (float)(ValZ - OldValZ) / (float)(iValZ - iOldValZ);
                                        for (int k = 0; k <= (iValZ-iOldValZ); k++)
                                                CALIBR_BUFFER_Z[ind_Data][iOldValZ + k] = OldValZ + (float)(k * delta);
                                }
                                OldValZ = ValZ;
                                iOldValZ = iValZ;
                        }
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Convert_Pixel_X()
{
        for (ind_Step = 0; ind_Step < 10 * con_TBL_Height; ind_Step ++)
                for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++)
                        CALIBR_BUFFER_X[ind_Data][ind_Step] = (short int)0xE000;
        short int ValX, OldValX, iValX, iOldValX;
        float Value_Um;
        delta_Um = con_X_Step_Delta_um;
        for (ind_Step = 0; ind_Step < con_DIAP_Z_Steps; ind_Step ++) {
                OldValX = 0;
                Value_Um = 0;
                for (int ind_Data = StrToInt(Edit_ZERO_Left->Text); ind_Data < con_TBL_Width; ind_Data ++) {
                        ValX = CALIBR_BUFFER_PXL[ind_Data][ind_Step];
                        iValX = ind_Data;
                        if (ValX != 0) {
                                if (OldValX != 0) {
                                        float delta = (float)(ValX - OldValX) / (iValX - iOldValX);
                                        float deltaX = (float)delta_Um / (iValX - iOldValX);
                                        for (int k = 0; k <= (iValX - iOldValX); k++)
                                                CALIBR_BUFFER_X[iOldValX + k][OldValX + (int)(k * delta)] = (float)((0x1000 * (Value_Um + deltaX * k)) / (HELLO_PARAMETERS.REG_NAME.RANGE_XE * 1000));
                                        Value_Um += delta_Um;
                                }
                                OldValX = ValX;
                                iOldValX = iValX;
                        }
                }
                Value_Um = 0;
                OldValX = 0;
                for (int ind_Data = StrToInt(Edit_ZERO_Right->Text); ind_Data > -1; ind_Data --) {
                        ValX = CALIBR_BUFFER_PXL[ind_Data][ind_Step];
                        iValX = ind_Data;
                        if (ValX != 0) {
                                if (OldValX != 0) {
                                        float delta = (float)(ValX - OldValX) / (iValX - iOldValX);
                                        float deltaX = (float)delta_Um / (iValX - iOldValX);
                                        for (int k = 0; k <= (iOldValX - iValX); k++)
                                                CALIBR_BUFFER_X[iOldValX - k][ValX + (int)(k * delta)] = (float)((0x1000 * (Value_Um + deltaX * k)) / (HELLO_PARAMETERS.REG_NAME.RANGE_XE * 1000));
                                        Value_Um -= delta_Um;
                                }
                                OldValX = ValX;
                                iOldValX = iValX;
                        }
                }
        }
        for (WORD ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++) {
                OldValX = (short int)0xE000;
                for (ind_Step = 0; ind_Step < 10 * con_TBL_Height; ind_Step ++) {
                        ValX = CALIBR_BUFFER_X[ind_Data][ind_Step];
                        iValX = ind_Step;
                        if (ValX != (short int)0xE000) {
                                if (OldValX != (short int)0xE000) {
                                        float delta = (float)(ValX - OldValX) / (iValX - iOldValX);
                                        for (int k = 0; k <= (iValX - iOldValX); k++)
                                                CALIBR_BUFFER_X[ind_Data][iOldValX + k] = OldValX + (float)(k * delta);
                                }
                                OldValX = ValX;
                                iOldValX = iValX;
                        }
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Timer_ErrorsTimer(TObject *Sender)
{
        AnsiString str = "";
        BOOL bNeedCloseConnect = false;
        Timer_Errors->Enabled = false;
        RF620INT_STATUS RF620_status_tmp = RF620_status;
        switch (RF620_status_tmp) {
                case RF620INT_PORT_NOT_FOUND:
                        str = "Port not found.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_PORT_NOT_OPENNED:
                        str = "Port not openned.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_READ_TIMEOUT:
                        str = "Read timeout.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_READ_ERROR:
                        str = "Read error.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_WRITE_TIMEOUT:
                        str = "Write timeout.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_WRITE_ERROR:
                        str = "Write error.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_OTHER_ERROR:
                        str = "Unknown error.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_PACKET_ERROR:
                        str = "Packet error.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_NOT_CONNECTED:
                        str = "Connection close.";
                        bNeedCloseConnect = true;
                break;
                case RF620INT_FLASH_ERROR:
                        str = "FLASH error.";
                        bNeedCloseConnect = true;
                break;
        }
        if (bNeedCloseConnect == true) {
                ComboBox_Connect_NetAddress->Clear();
                for (BYTE i = 0; i <= con_cNET_ADDR; i++)
                        ComboBox_Connect_NetAddress->AddItem(IntToStr(i), Sender);
                ComboBox_Connect_NetAddress->ItemIndex = 0;
                Form_Main->Constraints->MinHeight = (Form_Main->Height - (Panel_Parameters->Height + Panel_Connect->Height + Panel_Ranges->Height)) + (Panel_Connect->Height + Panel_Ranges->Height);
                Min_Height = Form_Main->Constraints->MinHeight;
                Form_Main->Height = Form_Main->Constraints->MinHeight;
                Timer_Resize->Enabled = false;
                Timer_Result->Enabled = false;
                Timer_Motor->Enabled = false;
                ind_User_Param = ind_Factory_Param = 0;
                fAngle = 0.0;
                Panel_Angle->Caption = "Angle = "+ FloatToStrF(fAngle * 180 / M_PI, ffFixed, 3, 3) + "";
                Label_Ranges_Model_Value->Caption = "none";
                Label_Ranges_Serial_Value->Caption = "none";
                Label_Ranges_Base_Value->Caption = "none mm";
                Label_Ranges_RangeZ_Value->Caption = "none mm";
                Label_Ranges_RangeXs_Value->Caption = "none mm";
                Label_Ranges_RangeXe_Value->Caption = "none mm";
                ComboBox_Connect_Baud->Enabled = true;
                ComboBox_Connect_NetAddress->Enabled = true;
                Button_Get_Result->Caption = "Start Measure";
                Button_Get_Result->Enabled = false;
                Button_Get_Video->Enabled = false;
                Button_Connect->Caption = "Connect";
                Timer_Repaint->Enabled = false;
                Timer_Video->Enabled = false;
                Button_Get_Video->Caption = "Get BITMAP";
        }
        if (RF620_status_tmp != RF620INT_OK) {
                Application->MessageBox(str.c_str(), "RF620-SP.", MB_ICONERROR);
                RF620_status = RF620INT_OK;
        }
        Timer_Errors->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::PaintBox_BitMapMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        if (Button == mbLeft) {
                ROI_Window_Old = ROI_Window;
                ROI_Window.Left = X;
                ROI_Window.Top = Y;
                ROI_Window.Right = X;
                ROI_Window.Bottom = Y;
                Set_ROI = true;
        } else if (Button == mbRight) {
                SaveDialogData->Filter = "All files(*.*)|*.*|BitMap files(*.bmp)|*.bmp";
                SaveDialogData->FilterIndex = 2;
                SaveDialogData->DefaultExt = "bmp";
                if (!DirectoryExists(CurrDirectory + "\\BMPFiles"))
                        CreateDir(CurrDirectory + "\\BMPFiles");
                SaveDialogData->InitialDir = CurrDirectory + "\\BMPFiles";
                if (SaveDialogData->Execute()) {
                        pBitMap->SaveToFile(SaveDialogData->FileName);
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::PaintBox_BitMapMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        WORD Value_Left, Value_Top, Value_Width, Value_Height;
        if ((Button == mbLeft) && (Set_ROI == true)) {
                if ((ROI_Window.Right < ROI_Window.Left) || (ROI_Window.Bottom < ROI_Window.Top)) {
                        ROI_Window.Left = 0;
                        ROI_Window.Top = 0;
                        ROI_Window.Right = con_BMP_Width;
                        ROI_Window.Bottom = con_BMP_Height;
                 } else {
                        ROI_Window.Right = X;
                        ROI_Window.Bottom = Y;
                }
                if (ROI_Window.Right > con_BMP_Width) ROI_Window.Right = con_BMP_Width;
                if (ROI_Window.Bottom > con_BMP_Height) ROI_Window.Bottom = con_BMP_Height;
                if (((ROI_Window.Right - ROI_Window.Left) < con_BMP_Width / 10) || ((ROI_Window.Bottom - ROI_Window.Top) < con_BMP_Height / 10))
                        ROI_Window = ROI_Window_Old;

                Value_Left = 100 * ROI_Window.Left / con_BMP_Width;
                Value_Width = 100 * ROI_Window.Width() / con_BMP_Width;
                if ((Value_Left + Value_Width) > 100) Value_Width = 100 - Value_Left;

                ValueListEditor_User_Parameters->Cells[1][ra_USER_WND_LW + 1] = (IntToStr(Value_Left) | IntToStr(Value_Width) << 7);

                Value_Top = 100 * ROI_Window.Top / con_BMP_Height;
                Value_Height = 100 * ROI_Window.Height() / con_BMP_Height;
                if ((Value_Top + Value_Height) > 100) Value_Height = 100 - Value_Top;

                ValueListEditor_User_Parameters->Cells[1][ra_USER_WND_TH + 1] = (IntToStr(Value_Top) | IntToStr(Value_Height) << 7);

                USER_PARAMETERS.REG_NAME.WINDOW_LW.BITs.LEFT = Value_Left;
                USER_PARAMETERS.REG_NAME.WINDOW_LW.BITs.WIDTH = Value_Width;
                USER_PARAMETERS.REG_NAME.WINDOW_TH.BITs.TOP= Value_Top;
                USER_PARAMETERS.REG_NAME.WINDOW_TH.BITs.HEIGHT = Value_Height;
                Set_ROI = false;
                if ((ROI_Window.Width() != 0) && (ROI_Window.Height() != 0))
                        FrameFreq = ((float) 1000 / (float) (0.000025 * (ROI_Window.Width() + 152) * ROI_Window.Height() + 0.1) + 0.5);
                Panel_Freq->Caption = "Freq.:" + IntToStr(FrameFreq) + " fps";
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::PaintBox_BitMapMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
        if (Set_ROI == true) {
                ROI_Window.Right = X;
                ROI_Window.Bottom = Y;
                if (ROI_Window.Right > con_BMP_Width) ROI_Window.Right = con_BMP_Width;
                if (ROI_Window.Bottom > con_BMP_Height) ROI_Window.Bottom = con_BMP_Height;
                if ((ROI_Window.Width() != 0) && (ROI_Window.Height() != 0))
                        FrameFreq = ((float) 1000 / (float) (0.000025 * (ROI_Window.Width() + 152) * ROI_Window.Height() + 0.1) + 0.5);
                Panel_Freq->Caption = "Freq.:" + IntToStr(FrameFreq) + " fps";
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Timer_VideoTimer(TObject *Sender)
{
        WORD Bmp_Left, Bmp_Top, Bmp_Width, Bmp_Height;
        Timer_Video->Enabled = false;
        RF620_status = RF620Interface->RF620_Get_BitMap(&BITMAP_BUFFER[0], &Bmp_Left, &Bmp_Top, &Bmp_Width, &Bmp_Height);
        if (RF620_status != RF620INT_OK) return;
        PaintBox_BitMap->Left = Bmp_Left;
        PaintBox_BitMap->Top = Bmp_Top;
        PaintBox_BitMap->Width = Bmp_Width;
        PaintBox_BitMap->Height = Bmp_Height;
        pBitMap->Width = Bmp_Width;
        pBitMap->Height = Bmp_Height;
        for (WORD y = 0; y < pBitMap->Height; y++) {
                BYTE *ptr_In = (BYTE*) pBitMap->ScanLine[y];
                BYTE *ptr_Out = &BITMAP_BUFFER[y * Bmp_Width];
                for (WORD x = 0; x < pBitMap->Width; x++)
                        ptr_In[x] = ptr_Out[x] << 1;
        }
        Timer_Video->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Calibr_CalculateClick(TObject *Sender)
{
        Convert_Line_Pixel();
        Convert_Pixel_Z();
        Convert_Pixel_X();
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::CSpinEdit_Calibr_Ind_StepChange(TObject *Sender)
{
        WORD ind_Data, ind_Step;
        Series_Calibr_Oscilloscope1->Clear();
        ind_Step = CSpinEdit_Calibr_Ind_Step->Value;
        for (ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++) {
                Series_Calibr_Oscilloscope1->AddXY(ind_Data, CALIBR_BUFFER[ind_Data][ind_Step], IntToStr(ind_Data), clGreen);
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::Button_Calibr_Rst_PntClick(TObject *Sender)
{
        WORD ind_Data, ind_Step;
        ind_Data = CSpinEdit_Calibr_Ind_Data->Value;
        ind_Step = CSpinEdit_Calibr_Ind_Step->Value;
        CALIBR_BUFFER[ind_Data][ind_Step] = 0;
        Series_Calibr_Oscilloscope1->Clear();
        for (ind_Data = 0; ind_Data < con_TBL_Width; ind_Data ++) {
                Series_Calibr_Oscilloscope1->AddXY(ind_Data, CALIBR_BUFFER[ind_Data][ind_Step], IntToStr(ind_Data), clGreen);
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::MenuItem4Click(TObject *Sender)
{
        int iFileHandle;
        int iFileLength;
        char ch;
        AnsiString str, X, Z, str1;
        SaveDialogData->Filter = "All files(*.*)|*.*|Text files(*.txt)|*.txt";
        SaveDialogData->FilterIndex = 2;
        SaveDialogData->DefaultExt = "txt";
        if (!DirectoryExists(CurrDirectory + "\\DataFiles"))
                CreateDir(CurrDirectory + "\\DataFiles");
        SaveDialogData->InitialDir = CurrDirectory + "\\DataFiles";
        if (SaveDialogData->Execute()) {
                iFileHandle = FileCreate(SaveDialogData->FileName);
                str += "N";
                while (str.Length() < 10) str += " ";
                str += "X";
                while (str.Length() < 30) str += " ";
                str += "Z";
                FileWrite(iFileHandle, str.c_str(), str.Length());
                ch=(char)13;
                FileWrite(iFileHandle, &ch, 1);
                ch=(char)10;
                FileWrite(iFileHandle, &ch, 1);
                for (WORD i = 0; i < DATA_TO_FILE_Pnt; i++) {
                        X = FloatToStrF(DATA_TO_FILE[i][0], ffFixed, 6, 2);
                        Z = FloatToStrF(DATA_TO_FILE[i][1], ffFixed, 6, 2);
                        str1 = "";
                        for (WORD j = 1; j <= X.Length(); j++) {
                                ch = X[j];
                                if (ch == ',') ch = '.';
                                str1 += ch;
                        }
                        X = str1;
                        str1 = "";
                        for (WORD j = 1; j <= Z.Length(); j++) {
                                ch = Z[j];
                                if (ch == ',') ch = '.';
                                str1 += ch;
                        }
                        Z = str1;
                        str = IntToStr(i + 1);
                        while (str.Length() < 10) str += " ";
                        str += X;
                        while (str.Length() < 30) str += " ";
                        str += Z;
                        FileWrite(iFileHandle, str.c_str(), str.Length());
                        ch=(char)13;
                        FileWrite(iFileHandle, &ch, 1);
                        ch=(char)10;
                        FileWrite(iFileHandle, &ch, 1);
                }
                FileClose(iFileHandle);
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm_Main::MenuItem5Click(TObject *Sender)
{
        Series_Result_Oscilloscope->Clear();
        for (WORD ind = 0; ind < 1024; ind ++)
                DATA_TO_FILE[ind][0] = DATA_TO_FILE[ind][1] = 0;
}
//---------------------------------------------------------------------------
